@@ -15,122 +15,35 @@
 
 """Rally command: task"""
 
-from __future__ import print_function
-import collections
-import datetime as dt
 import itertools
 import json
 import os
 import sys
 import webbrowser
 
-import jsonschema
-import six
-
 from rally.cli import cliutils
 from rally.cli import envutils
-from rally.common import fileutils
+from rally.cli import task_results_loader
+from rally.cli import yamlutils as yaml
 from rally.common import logging
 from rally.common import utils as rutils
 from rally.common import version
-from rally.common import yamlutils as yaml
 from rally import consts
 from rally import exceptions
 from rally import plugins
 from rally.task import atomic
 from rally.task.processing import charts
-from rally.task import utils as tutils
 from rally.utils import strutils
 
 
 LOG = logging.getLogger(__name__)
 
-OLD_TASK_RESULT_SCHEMA = {
-    "type": "object",
-    "$schema": consts.JSON_SCHEMA,
-    "properties": {
-        "key": {
-            "type": "object",
-            "properties": {
-                "kw": {
-                    "type": "object"
-                },
-                "name": {
-                    "type": "string"
-                },
-                "pos": {
-                    "type": "integer"
-                },
-            },
-            "required": ["kw", "name", "pos"]
-        },
-        "sla": {
-            "type": "array",
-            "items": {
-                "type": "object",
-                "properties": {
-                    "criterion": {
-                        "type": "string"
-                    },
-                    "detail": {
-                        "type": "string"
-                    },
-                    "success": {
-                        "type": "boolean"
-                    }
-                }
-            }
-        },
-        "hooks": {"type": "array"},
-        "result": {
-            "type": "array",
-            "items": {
-                "type": "object",
-                "properties": {
-                    "atomic_actions": {
-                        "type": "object"
-                    },
-                    "duration": {
-                        "type": "number"
-                    },
-                    "error": {
-                        "type": "array"
-                    },
-                    "idle_duration": {
-                        "type": "number"
-                    },
-                    "output": {"type": "object"}
-                },
-                "required": ["atomic_actions", "duration", "error",
-                             "idle_duration"]
-            },
-            "minItems": 0
-        },
-        "load_duration": {
-            "type": "number",
-        },
-        "full_duration": {
-            "type": "number",
-        },
-        "created_at": {
-            "type": "string"
-        }
-    },
-    "required": ["key", "sla", "result", "load_duration", "full_duration"],
-    "additionalProperties": False
-}
-
 
 class FailedToLoadTask(exceptions.RallyException):
     error_code = 117
     msg_fmt = "Invalid %(source)s passed:\n\n\t %(msg)s"
 
 
-class FailedToLoadResults(exceptions.RallyException):
-    error_code = 225
-    msg_fmt = "ERROR: Invalid task result format in %(source)s\n\n\t%(msg)s"
-
-
 class TaskCommands(object):
     """Set of commands that allow you to manage tasks and results.
 
@@ -179,7 +92,7 @@ class TaskCommands(object):
         if raw_args:
             try:
                 data = yaml.safe_load(raw_args)
-                if isinstance(data, (six.text_type, six.string_types)):
+                if isinstance(data, str):
                     raise yaml.ParserError("String '%s' doesn't look like a "
                                            "dictionary." % raw_args)
                 task_args.update(data)
@@ -288,6 +201,12 @@ class TaskCommands(object):
                                                   args_file=task_args_file)
         print("Running Rally version", version.version_string())
 
+        return self._start_task(api, deployment, task_config=input_task,
+                                tags=tags, do_use=do_use,
+                                abort_on_sla_failure=abort_on_sla_failure)
+
+    def _start_task(self, api, deployment, task_config, tags=None,
+                    do_use=False, abort_on_sla_failure=False):
         try:
             task_instance = api.task.create(deployment=deployment, tags=tags)
             tags = "[tags: '%s']" % "', '".join(tags) if tags else ""
@@ -302,7 +221,7 @@ class TaskCommands(object):
             if do_use:
                 self.use(api, task_instance["uuid"])
 
-            api.task.start(deployment=deployment, config=input_task,
+            api.task.start(deployment=deployment, config=task_config,
                            task=task_instance["uuid"],
                            abort_on_sla_failure=abort_on_sla_failure)
 
@@ -314,6 +233,72 @@ class TaskCommands(object):
             return 2
         return 0
 
+    @cliutils.args("--deployment", dest="deployment", type=str,
+                   metavar="<uuid>", required=False,
+                   help="UUID or name of a deployment.")
+    @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task.")
+    @cliutils.args("--scenario", type=str, dest="scenarios", nargs="+",
+                   help="scenario name of workload")
+    @cliutils.args("--tag", nargs="+", dest="tags", type=str, required=False,
+                   help="Mark the task with a tag or a few tags.")
+    @cliutils.args("--no-use", action="store_false", dest="do_use",
+                   help="Don't set new task as default for future operations.")
+    @cliutils.args("--abort-on-sla-failure", action="store_true",
+                   dest="abort_on_sla_failure",
+                   help="Abort the execution of a task when any SLA check "
+                        "for it fails for subtask or workload.")
+    @envutils.with_default_deployment(cli_arg_name="deployment")
+    @envutils.with_default_task_id
+    @plugins.ensure_plugins_are_loaded
+    def restart(self, api, deployment=None, task_id=None, scenarios=None,
+                tags=None, do_use=False, abort_on_sla_failure=False):
+        """Restart a task or some scenarios in workloads of task."""
+        if scenarios is not None:
+            scenarios = (isinstance(scenarios, list) and scenarios
+                         or [scenarios])
+        task = api.task.get(task_id=task_id, detailed=True)
+        if task["status"] == consts.TaskStatus.CRASHED or task["status"] == (
+                consts.TaskStatus.VALIDATION_FAILED):
+            print("-" * 80)
+            print("\nUnable to restart task.")
+            validation = task["validation_result"]
+            if logging.is_debug():
+                print(yaml.safe_load(validation["trace"]))
+            else:
+                print(validation["etype"])
+                print(validation["msg"])
+                print("\nFor more details run:\nrally -d task detailed %s"
+                      % task["uuid"])
+            return 1
+        retask = {"version": 2, "title": task["title"],
+                  "description": task["description"],
+                  "tags": task["tags"], "subtasks": []}
+        for subtask in task["subtasks"]:
+            workloads = []
+            for workload in subtask["workloads"]:
+                if scenarios is None or workload["name"] in scenarios:
+                    workloads.append({
+                        "scenario": {workload["name"]: workload["args"]},
+                        "contexts": workload["contexts"],
+                        "runner": {
+                            workload["runner_type"]: workload["runner"]},
+                        "hooks": workload["hooks"],
+                        "sla": workload["sla"]
+                    })
+            if workloads:
+                retask["subtasks"].append({
+                    "title": subtask["title"],
+                    "description": subtask["description"],
+                    "workloads": workloads})
+
+        if retask["subtasks"]:
+            return self._start_task(api, deployment, retask, tags=tags,
+                                    do_use=do_use,
+                                    abort_on_sla_failure=abort_on_sla_failure)
+        else:
+            print("Not Found matched scenario.")
+            return 1
+
     @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task.")
     @envutils.with_default_task_id
     @cliutils.args(
@@ -546,70 +531,16 @@ class TaskCommands(object):
     @envutils.with_default_task_id
     @cliutils.suppress_warnings
     def results(self, api, task_id=None):
-        """Display raw task results.
-
-        This will produce a lot of output data about every iteration.
-        """
-
-        task = api.task.get(task_id=task_id, detailed=True)
-        finished_statuses = (consts.TaskStatus.FINISHED,
-                             consts.TaskStatus.ABORTED)
-        if task["status"] not in finished_statuses:
-            print("Task status is %s. Results available when it is one of %s."
-                  % (task["status"], ", ".join(finished_statuses)))
+        """DEPRECATED since Rally 3.0.0."""
+        LOG.warning("CLI method `rally task results` is deprecated since "
+                    "Rally 3.0.0 and will be removed soon. "
+                    "Use `rally task report --json` instead.")
+        try:
+            self.export(api, tasks=[task_id], output_type="old-json-results")
+        except exceptions.RallyException as e:
+            print(e.format_message())
             return 1
 
-        # TODO(chenhb): Ensure `rally task results` puts out old format.
-        for workload in itertools.chain(
-                *[s["workloads"] for s in task["subtasks"]]):
-            for itr in workload["data"]:
-                itr["atomic_actions"] = collections.OrderedDict(
-                    tutils.WrapperForAtomicActions(
-                        itr["atomic_actions"]).items()
-                )
-
-        results = []
-        for w in itertools.chain(*[s["workloads"] for s in task["subtasks"]]):
-            w["runner"]["type"] = w["runner_type"]
-
-            def port_hook_cfg(h):
-                h["config"] = {
-                    "name": h["config"]["action"][0],
-                    "args": h["config"]["action"][1],
-                    "description": h["config"].get("description", ""),
-                    "trigger": {"name": h["config"]["trigger"][0],
-                                "args": h["config"]["trigger"][1]}
-                }
-                return h
-
-            hooks = [port_hook_cfg(h) for h in w["hooks"]]
-
-            created_at = dt.datetime.strptime(w["created_at"],
-                                              "%Y-%m-%dT%H:%M:%S")
-            created_at = created_at.strftime("%Y-%d-%mT%H:%M:%S")
-
-            results.append({
-                "key": {
-                    "name": w["name"],
-                    "description": w["description"],
-                    "pos": w["position"],
-                    "kw": {
-                        "args": w["args"],
-                        "runner": w["runner"],
-                        "context": w["contexts"],
-                        "sla": w["sla"],
-                        "hooks": [h["config"] for h in w["hooks"]],
-                    }
-                },
-                "result": w["data"],
-                "sla": w["sla_results"].get("sla", []),
-                "hooks": hooks,
-                "load_duration": w["load_duration"],
-                "full_duration": w["full_duration"],
-                "created_at": created_at})
-
-        print(json.dumps(results, sort_keys=False, indent=4))
-
     @cliutils.args("--deployment", dest="deployment", type=str,
                    metavar="<uuid>", required=False,
                    help="UUID or name of a deployment.")
@@ -682,137 +613,6 @@ class TaskCommands(object):
                 print("There are no tasks. To run a new task, use:\n"
                       "\trally task start")
 
-    def _load_task_results_file(self, api, task_id):
-        """Load the json file which is created by `rally task results`"""
-
-        with open(os.path.expanduser(task_id)) as inp_js:
-            tasks_results = json.loads(inp_js.read())
-
-        if isinstance(tasks_results, list):
-            # it is an old format:
-
-            task = {"version": 2,
-                    "title": "Task loaded from a file.",
-                    "description": "Auto-ported from task format V1.",
-                    "uuid": "n/a",
-                    "env_name": "n/a",
-                    "env_uuid": "n/a",
-                    "tags": [],
-                    "status": consts.TaskStatus.FINISHED,
-                    "subtasks": []}
-
-            start_time = None
-
-            for result in tasks_results:
-                try:
-                    jsonschema.validate(
-                        result, OLD_TASK_RESULT_SCHEMA)
-                except jsonschema.ValidationError as e:
-                    raise FailedToLoadResults(source=task_id,
-                                              msg=six.text_type(e))
-
-                iter_count = 0
-                failed_iter_count = 0
-                min_duration = None
-                max_duration = None
-
-                for itr in result["result"]:
-                    if start_time is None or itr["timestamp"] < start_time:
-                        start_time = itr["timestamp"]
-                    # NOTE(chenhb): back compatible for atomic_actions
-                    itr["atomic_actions"] = list(
-                        tutils.WrapperForAtomicActions(itr["atomic_actions"],
-                                                       itr["timestamp"]))
-
-                    iter_count += 1
-                    if itr.get("error"):
-                        failed_iter_count += 1
-
-                    duration = itr.get("duration", 0)
-
-                    if max_duration is None or duration > max_duration:
-                        max_duration = duration
-
-                    if min_duration is None or min_duration > duration:
-                        min_duration = duration
-
-                durations_stat = charts.MainStatsTable(
-                    {"total_iteration_count": iter_count})
-
-                for itr in result["result"]:
-                    durations_stat.add_iteration(itr)
-
-                created_at = dt.datetime.strptime(result["created_at"],
-                                                  "%Y-%d-%mT%H:%M:%S")
-                updated_at = created_at + dt.timedelta(
-                    seconds=result["full_duration"])
-                created_at = created_at.strftime(consts.TimeFormat.ISO8601)
-                updated_at = updated_at.strftime(consts.TimeFormat.ISO8601)
-                pass_sla = all(s.get("success") for s in result["sla"])
-                runner_type = result["key"]["kw"]["runner"].pop("type")
-                for h in result["hooks"]:
-                    trigger = h["config"]["trigger"]
-                    h["config"] = {
-                        "description": h["config"].get("description"),
-                        "action": (h["config"]["name"], h["config"]["args"]),
-                        "trigger": (trigger["name"], trigger["args"])}
-                workload = {"uuid": "n/a",
-                            "name": result["key"]["name"],
-                            "position": result["key"]["pos"],
-                            "description": result["key"].get("description",
-                                                             ""),
-                            "full_duration": result["full_duration"],
-                            "load_duration": result["load_duration"],
-                            "total_iteration_count": iter_count,
-                            "failed_iteration_count": failed_iter_count,
-                            "min_duration": min_duration,
-                            "max_duration": max_duration,
-                            "start_time": start_time,
-                            "created_at": created_at,
-                            "updated_at": updated_at,
-                            "args": result["key"]["kw"]["args"],
-                            "runner_type": runner_type,
-                            "runner": result["key"]["kw"]["runner"],
-                            "hooks": result["hooks"],
-                            "sla": result["key"]["kw"]["sla"],
-                            "sla_results": {"sla": result["sla"]},
-                            "pass_sla": pass_sla,
-                            "contexts": result["key"]["kw"]["context"],
-                            "contexts_results": [],
-                            "data": sorted(result["result"],
-                                           key=lambda x: x["timestamp"]),
-                            "statistics": {
-                                "durations": durations_stat.to_dict()},
-                            }
-                task["subtasks"].append(
-                    {"title": "A SubTask",
-                     "description": "",
-                     "workloads": [workload]})
-            return [task]
-        elif isinstance(tasks_results, dict) and "tasks" in tasks_results:
-            for task_result in tasks_results["tasks"]:
-                try:
-                    jsonschema.validate(task_result,
-                                        api.task.TASK_SCHEMA)
-                except jsonschema.ValidationError as e:
-                    msg = six.text_type(e)
-                    raise exceptions.RallyException(
-                        "ERROR: Invalid task result format\n\n\t%s" % msg)
-                task_result.setdefault("env_name", "n/a")
-                task_result.setdefault("env_uuid", "n/a")
-                for subtask in task_result["subtasks"]:
-                    for workload in subtask["workloads"]:
-                        workload.setdefault("contexts_results", [])
-                        workload["runner_type"], workload["runner"] = list(
-                            workload["runner"].items())[0]
-                        workload["name"], workload["args"] = list(
-                            workload.pop("scenario").items())[0]
-
-            return tasks_results["tasks"]
-        else:
-            raise FailedToLoadResults(
-                source=task_id, msg="Wrong format")
-
     @cliutils.args("--out", metavar="<path>",
                    type=str, dest="out", required=False,
                    help="Path to output file.")
@@ -916,6 +716,8 @@ class TaskCommands(object):
         else:
             cliutils.print_list(data, ("benchmark", "pos", "criterion",
                                        "status", "detail"))
+        if not data:
+            return 2
         return failed_criteria
 
     @cliutils.args("--uuid", type=str, dest="task_id",
@@ -925,7 +727,7 @@ class TaskCommands(object):
 
         print("Using task: %s" % task_id)
         api.task.get(task_id=task_id)
-        fileutils.update_globals_file("RALLY_TASK", task_id)
+        envutils.update_globals_file("RALLY_TASK", task_id)
 
     @cliutils.args("--uuid", dest="tasks", nargs="+", type=str,
                    help="UUIDs of tasks or json reports of tasks")
@@ -962,7 +764,7 @@ class TaskCommands(object):
         for task_file_or_uuid in tasks:
             if os.path.exists(os.path.expanduser(task_file_or_uuid)):
                 exported_tasks.extend(
-                    self._load_task_results_file(api, task_file_or_uuid)
+                    task_results_loader.load(task_file_or_uuid)
                 )
             else:
                 exported_tasks.append(task_file_or_uuid)
@@ -1019,7 +821,7 @@ class TaskCommands(object):
         """Import json results of a test into rally database"""
 
         if os.path.exists(os.path.expanduser(task_file)):
-            tasks_results = self._load_task_results_file(api, task_file)
+            tasks_results = task_results_loader.load(task_file)
             for task_results in tasks_results:
                 task = api.task.import_results(deployment=deployment,
                                                task_results=task_results,
