@@ -12,16 +12,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import print_function
 import collections
+import queue as Queue
 import string
 import sys
 import threading
 import time
+from unittest import mock
 
 import ddt
-import mock
-from six.moves import queue as Queue
 import testtools
 
 from rally.common import utils
@@ -150,29 +149,6 @@ class FirstIndexTestCase(test.TestCase):
         self.assertIsNone(utils.first_index(lst, lambda e: e == 2))
 
 
-class EditDistanceTestCase(test.TestCase):
-
-    def test_distance_empty_strings(self):
-        dist = utils.distance("", "")
-        self.assertEqual(0, dist)
-
-    def test_distance_equal_strings(self):
-        dist = utils.distance("abcde", "abcde")
-        self.assertEqual(0, dist)
-
-    def test_distance_replacement(self):
-        dist = utils.distance("abcde", "__cde")
-        self.assertEqual(2, dist)
-
-    def test_distance_insertion(self):
-        dist = utils.distance("abcde", "ab__cde")
-        self.assertEqual(2, dist)
-
-    def test_distance_deletion(self):
-        dist = utils.distance("abcde", "abc")
-        self.assertEqual(2, dist)
-
-
 class TenantIteratorTestCase(test.TestCase):
 
     def test_iterate_per_tenant(self):
@@ -338,9 +314,9 @@ class RandomNameTestCase(test.TestCase):
         name = "foo"
         self.assertFalse(utils.name_matches_object(name, One, Two))
         # ensure that exactly one of the two objects is checked
-        self.assertItemsEqual(
-            One.name_matches_object.call_args_list +
-            Two.name_matches_object.call_args_list,
+        self.assertCountEqual(
+            One.name_matches_object.call_args_list
+            + Two.name_matches_object.call_args_list,
             [mock.call(name)])
 
     def test_name_matches_object_differing_list(self):
@@ -559,45 +535,6 @@ class LockedDictTestCase(test.TestCase):
         self.assertEqual({"memo": "foo_memo"}, kw)
 
 
-@ddt.ddt
-class FloatFormatterTestCase(test.TestCase):
-
-    @ddt.data(
-        {
-            "num_float": 0,
-            "num_str": "0.0"
-        },
-        {
-            "num_float": 37,
-            "num_str": "37.0"
-        },
-        {
-            "num_float": 0.0000001,
-            "num_str": "0.0"
-        },
-        {
-            "num_float": 0.000000,
-            "num_str": "0.0"
-        },
-        {
-            "num_float": 1.0000001,
-            "num_str": "1.0"
-        },
-        {
-            "num_float": 1.0000011,
-            "num_str": "1.000001"
-        },
-        {
-            "num_float": 1.0000019,
-            "num_str": "1.000002"
-        }
-
-    )
-    @ddt.unpack
-    def test_format_float_to_str(self, num_float, num_str):
-        self.assertEqual(num_str, utils.format_float_to_str(num_float))
-
-
 class DequeAsQueueTestCase(test.TestCase):
 
     def setUp(self):
